# Copyright (c) 2016 Red Hat, Inc.
# Author: Stanislav Kontar, Red Hat Product Security
# License: LGPLv3+

"""
Implements class for CVSS2 specification as defined at
https://www.first.org/cvss/v2/guide .

The library is compatible with both Python 2 and Python 3.
"""

from __future__ import unicode_literals

from decimal import Decimal as D, ROUND_HALF_UP

from .constants2 import METRICS_ABBREVIATIONS, METRICS_MANDATORY, METRICS_VALUES
from .exceptions import CVSS2MalformedError, CVSS2MandatoryError, CVSS2RHMalformedError, \
    CVSS2RHScoreDoesNotMatch


def round_to_1_decimal(value):
    """
    Round to one decimal.
    """
    return value.quantize(D('0.1'), rounding=ROUND_HALF_UP)


class CVSS2(object):
    """
    Class to hold CVSS2 vector, parsed values, and all scores.
    """
    @classmethod
    def from_rh_vector(cls, vector):
        """
        Creates a CVSS2 object from CVSS vector in Red Hat notation, e.g. containing base score.
        Also checks if the score matches the vector.

        Args:
            vector (str): string specifying CVSS3 vector in Red Hat notation, fields may be out of
                          order, fields which are not mandatory may be missing

        Returns:
            CVSS2: the generated CVSS2 object created from the vector string

        Raises:
            CVSS2RHMalformedError: if vector is not in expected format for Red Hat notation
            CVSS2RHScoreDoesNotMatch: if vector and score do not match
        """
        try:
            score, base_vector = vector.split('/', 1)
        except ValueError:
            raise CVSS2RHMalformedError('Malformed CVSS2 vector in Red Hat notation "{0}"'
                                        .format(vector))
        try:
            score_value = float(score)
        except ValueError:
            raise CVSS2RHMalformedError('Malformed CVSS2 vector in Red Hat notation "{0}"'
                                        .format(vector))
        cvss_object = cls(base_vector)
        if cvss_object.scores()[0] == score_value:
            return cvss_object
        else:
            raise CVSS2RHScoreDoesNotMatch('CVSS2 vector in Red Hat notation "{0}" has score of '
                                           '"{1}" which does not match specified score of "{2}"'
                                           .format(base_vector, cvss_object.scores()[0],
                                                   score))

    def __init__(self, vector):
        """
        Args:
            vector (str): string specifying CVSS2 vector, fields may be out of order, fields which
                          are not mandatory may be missing
        """
        self.vector = vector
        self.metrics = {}

        self.base_score = None
        self.temporal_score = None
        self.environmental_score = None

        self.parse_vector()
        self.check_mandatory()
        self.compute_base_score()
        self.compute_temporal_score()
        self.compute_environmental_score()

    def parse_vector(self):
        """
        Parses metrics from the CVSS2 vector.

        Raises:
            CVSS2MalformedError: if vector is not in expected format
        """
        if self.vector == '':
            raise CVSS2MalformedError('Malformed CVSS2 vector, vector is empty')

        if self.vector.endswith('/'):
            raise CVSS2MalformedError('Malformed CVSS2 vector, trailing "/"')

        fields = self.vector.split('/')

        # Parse fields
        for field in fields:
            if field == '':
                raise CVSS2MalformedError('Empty field in CVSS2 vector "{0}"'.format(self.vector))

            try:
                metric, value = field.split(':')
            except ValueError:
                raise CVSS2MalformedError('Malformed CVSS2 field "{0}"'.format(field))

            if metric in METRICS_ABBREVIATIONS:
                if value in METRICS_VALUES[metric]:
                    if metric in self.metrics:
                        raise CVSS2MalformedError('Duplicate metric "{0}"'.format(metric))
                    self.metrics[metric] = value
                else:
                    raise CVSS2MalformedError('Unknown value "{0}" in field "{1}"'.format(value,
                                                                                          field))
            else:
                raise CVSS2MalformedError('Unknown metric "{0}" in field "{1}"'.format(metric,
                                                                                       field))

    def check_mandatory(self):
        """
        Checks if mandatory fields are in CVSS2 vector.

        Raises:
            CVSS2MandatoryError: if mandatory metric is missing in the vector
        """
        missing = []
        for mandatory_metric in METRICS_MANDATORY:
            if mandatory_metric not in self.metrics:
                missing.append(mandatory_metric)
        if missing:
            raise CVSS2MandatoryError('Missing mandatory metrics "{0}"'.format(', '.join(missing)))

    def get_value(self, abbreviation):
        """
        Gets value of specific metric specified by its abbreviation.
        """
        string_value = self.metrics.get(abbreviation, 'ND')
        result = METRICS_VALUES[abbreviation][string_value]
        return result

    def impact_equation(self):
        """
        Impact = 10.41*(1-(1-ConfImpact)*(1-IntegImpact)*(1-AvailImpact))
        """
        return D('10.41') * (D('1') - (D('1') - self.get_value('C')) *
                                      (D('1') - self.get_value('I')) *
                                      (D('1') - self.get_value('A')))

    def adjusted_impact_equation(self):
        """
        AdjustedImpact = min(10,10.41*(1-(1-ConfImpact*ConfReq)*(1-IntegImpact*IntegReq)
                                     *(1-AvailImpact*AvailReq)))
        """
        return min(D('10'),
                   D('10.41') * (D('1') - (D('1') - self.get_value('C') * self.get_value('CR')) *
                                          (D('1') - self.get_value('I') * self.get_value('IR')) *
                                          (D('1') - self.get_value('A') * self.get_value('AR')))
                   )

    def base_score_equation(self, adjusted_impact=False):
        """
        BaseScore = round_to_1_decimal(((0.6*Impact)+(0.4*Exploitability)-1.5)*f(Impact))
        Impact = see impact_equation or modified_impact_equation
        Exploitability = 20*AccessVector*AccessComplexity*Authentication
        f(impact)= 0 if Impact=0, 1.176 otherwise
        """
        if adjusted_impact:
            impact = self.adjusted_impact_equation()
        else:
            impact = self.impact_equation()

        exploitability = (D('20') * self.get_value('AV') * self.get_value('AC') *
                          self.get_value('Au'))
        f_impact = D('0') if impact == D('0') else D('1.176')
        return round_to_1_decimal(((D('0.6') * impact) + (D('0.4') * exploitability) -
                                   D('1.5')) * f_impact)

    def compute_base_score(self):
        """
        Compute base score using normal Impact equation. Do not allow negative result.
        """
        self.base_score = max(D('0.0'), self.base_score_equation())

    def temporal_score_equation(self, adjusted_impact=False):
        """
        TemporalScore = round_to_1_decimal(BaseScore*Exploitability
                        *RemediationLevel*ReportConfidence)
        """
        if adjusted_impact:
            base_score = self.base_score_equation(adjusted_impact=True)
        else:
            base_score = self.base_score
        return round_to_1_decimal(base_score * self.get_value('E') * self.get_value('RL') *
                                  self.get_value('RC'))

    def compute_temporal_score(self):
        """
        Compute temporal score using normal Impact equation.
        """
        if all(self.metrics.get(a, 'ND') == 'ND' for a in ['E', 'RL', 'RC']):
            self.temporal_score = None
        else:
            self.temporal_score = max(D('0.0'), self.temporal_score_equation())

    def compute_environmental_score(self):
        """
        EnvironmentalScore = round_to_1_decimal((AdjustedTemporal+
        (10-AdjustedTemporal)*CollateralDamagePotential)*TargetDistribution)

        AdjustedTemporal = TemporalScore recomputed with the BaseScores Impact sub-equation
                           replaced with the AdjustedImpact equation
        """
        if all(self.metrics.get(a, 'ND') == 'ND' for a in ['CDP', 'TD', 'CR', 'IR', 'AR']):
            self.environmental_score = None
        else:
            temporal_score_adjusted = self.temporal_score_equation(adjusted_impact=True)
            raw_environmental_score = round_to_1_decimal((temporal_score_adjusted +
                                                          (D('10') - temporal_score_adjusted) *
                                                          self.get_value('CDP')) *
                                                         self.get_value('TD'))
            self.environmental_score = max(D('0.0'), raw_environmental_score)

    def scores(self):
        """
        Returns all computed scores.

        Returns:
            (tuple): Base Score, Temporal Score, Environmental Score, either float or None if not
                     defined
        """
        scores = (self.base_score, self.temporal_score, self.environmental_score)
        return tuple(float(a) if a is not None else None for a in scores)

    def clean_vector(self):
        """
        Returns vector without optional metrics marked as ND and in preferred order.

        Returns:
            (str): cleaned CVSS2 with metrics in correct order
        """
        vector = []
        for metric in METRICS_ABBREVIATIONS:
            if metric in self.metrics:
                value = self.metrics[metric]
                if value != 'ND':
                    vector.append('{0}:{1}'.format(metric, value))
        return '/'.join(vector)

    def rh_vector(self):
        """
        Returns cleaned vector with score in Red Hat notation, e.g. score/vector.

        Example: 5.0/AV:L/AC:L/Au:M/C:N/I:P/A:C/E:U/RL:W/CDP:L/TD:H/AR:M
        """
        return str(self.scores()[0]) + '/' + self.clean_vector()

    def __eq__(self, o):
        if isinstance(o, CVSS2):
            return self.clean_vector().__eq__(o.clean_vector())
        return NotImplemented

    def __hash__(self):
        return hash(self.clean_vector())



# Copyright (c) 2016 Red Hat, Inc.
# Author: Stanislav Kontar, Red Hat Product Security
# License: LGPLv3+

from .cvss2 import CVSS2
from .cvss3 import CVSS3
from .exceptions import CVSSError, CVSS2Error, CVSS3Error
from .interactive import ask_interactively


__version__ = '2.1'



import re

from .cvss2 import CVSS2
from .cvss3 import CVSS3
from .exceptions import CVSSError


def parse_cvss_from_text(text):
    """
    Parses CVSS2 and CVSS3 vectors from arbitrary text and returns a list of CVSS objects.

    Parses text for substrings that look similar to CVSS vector
    and feeds these matches to CVSS constructor.

    Args:
        text (str): arbitrary text
    Returns:
        A list of CVSS objects.
    """
    # Looks for substrings which resemble CVSS2 or CVSS3 vectors.
    # CVSS3 vector starts with 'CVSS:3.x/' prefix - matched by non-capturing group.
    # Minimum vector length is 26.
    matches = re.compile(r'(?:CVSS:3\.\d/)?[A-Za-z:/]{26,}').findall(text)

    cvsss = set()
    for match in matches:
        try:
            if match.startswith('CVSS:3.'):
                cvss = CVSS3(match)
            else:
                cvss = CVSS2(match)

            cvsss.add(cvss)
        except (CVSSError, KeyError):
            pass

    return list(cvsss)



# Copyright (c) 2016 Red Hat, Inc.
# Author: Stanislav Kontar, Red Hat Product Security
# License: LGPLv3+

"""
Functions for interactively building CVSS vector string.

Compatible with both Python 2 and Python 3.
"""

from __future__ import print_function, unicode_literals


try:
    # noinspection PyUnresolvedReferences
    string_input = raw_input
except NameError:
    string_input = input


def color(text):
    """
    Replaces text in brackets with yellow bold variant for terminal. Also replaces | symbol with
    blue one. Should improve readability. Uses ANSI Styling codes.
    """
    colored = text.replace('(', '\033[33m\033[1m').replace(')', '\033[0m')
    colored = colored.replace('|', '\033[94m\033[1m|\033[0m')
    return colored


def ask_interactively(version=3.1, all_metrics=False, no_colors=False):
    """
    Asks user to build CVSS vector string interactively.

    Args:
        version (float): 2 or 3.0/3.1 for CVSS2 or CVSS3 respectively
        all_metrics (bool): If true, temporal and environmental metrics are asked, else only base
                            metrics are asked for
        no_colors (bool): If true, terminal coloring is not used in interactive mode

    Returns:
        (str): CVSS vector
    """
    # Import correct constants
    if version == 2:
        print('Interactive CVSS2 calculator')
        from .constants2 import METRICS_ABBREVIATIONS, METRICS_MANDATORY, METRICS_VALUE_NAMES
    elif version >= 3.0:
        print('Interactive CVSS3 calculator')
        from .constants3 import METRICS_ABBREVIATIONS, METRICS_MANDATORY, METRICS_VALUE_NAMES
    else:
        raise ValueError('Unknown version: {0}'.format(version))
    print()

    vector = []

    if all_metrics:
        metrics = METRICS_ABBREVIATIONS.keys()
    else:
        metrics = METRICS_MANDATORY

    for metric in metrics:
        # Print full metric name
        print(METRICS_ABBREVIATIONS[metric] + ':', end=' ')

        # Create metric value names with hints
        values = METRICS_VALUE_NAMES[metric]
        value_names = []
        for value in values:
            name = METRICS_VALUE_NAMES[metric][value]
            name_with_hints = name
            for letter in value:
                name_with_hints = name_with_hints.replace(letter, '(' + letter + ')')

            # Exceptions for hints
            if version >= 3.0 and name_with_hints == 'Not Defined':
                name_with_hints = '(X)Not Defined'
            elif version < 3.0:
                name_with_hints = {
                    '(P)roof-of-(C)oncept': '(P)roof-(O)f-(C)oncept',
                    '(U)nconfirmed': '(U)n(C)onfirmed',
                    '(U)ncorroborated': '(U)nco(R)roborated',
                }.get(name_with_hints, name_with_hints)

            value_names.append(name_with_hints)

        # Print value names with hints
        value_names_string = ' | '.join(value_names)
        if no_colors:
            print(value_names_string)
        else:
            print(color(value_names_string))

        # Ask for input
        while True:
            print(METRICS_ABBREVIATIONS[metric] + ':', end=' ')
            print('/'.join(values), end=' ')
            input_value = string_input().upper()
            if not input_value:
                if version == 2:
                    input_value = 'ND'
                else:
                    input_value = 'X'
            if input_value in values:
                vector.append(metric + ':' + input_value)
                break
        print()

    if version == 3.0:
        vector_string = 'CVSS:3.0/' + '/'.join(vector)
    elif version == 3.1:
        vector_string = 'CVSS:3.1/' + '/'.join(vector)
    else:
        vector_string = '/'.join(vector)
    return vector_string



# Copyright (c) 2016 Red Hat, Inc.
# Author: Stanislav Kontar, Red Hat Product Security
# License: LGPLv3+

"""
Implements class for CVSS3 specification as defined at
https://www.first.org/cvss/specification-document .

The library is compatible with both Python 2 and Python 3.
"""

from __future__ import unicode_literals

import copy
from decimal import Decimal as D, ROUND_CEILING

from .constants3 import METRICS_ABBREVIATIONS, METRICS_MANDATORY, METRICS_VALUES
from .exceptions import CVSS3MalformedError, CVSS3MandatoryError, CVSS3RHMalformedError, \
    CVSS3RHScoreDoesNotMatch


def round_up(value):
    """
    Round up is defined as the smallest number, specified to one decimal place, that is equal to
    or higher than its input. For example, Round up (4.02) is 4.1; and Round up (4.00) is 4.0.
    """
    return value.quantize(D('0.1'), rounding=ROUND_CEILING)


class CVSS3(object):
    """
    Class to hold CVSS3 vector, parsed values, and all scores.
    """
    @classmethod
    def from_rh_vector(cls, vector):
        """
        Creates a CVSS3 object from CVSS vector in Red Hat notation, e.g. containing base score.
        Also checks if the score matches the vector.

        Args:
            vector (str): string specifying CVSS3 vector in Red Hat notation, fields may be out of
                          order, fields which are not mandatory may be missing

        Returns:
            CVSS3: the generated CVSS3 object created from the vector string

        Raises:
            CVSS3RHMalformedError: if vector is not in expected format for Red Hat notation
            CVSS3RHScoreDoesNotMatch: if vector and score do not match
        """
        try:
            score, base_vector = vector.split('/', 1)
        except ValueError:
            raise CVSS3RHMalformedError('Malformed CVSS3 vector in Red Hat notation "{0}"'
                                        .format(vector))
        try:
            score_value = float(score)
        except ValueError:
            raise CVSS3RHMalformedError('Malformed CVSS3 vector in Red Hat notation "{0}"'
                                        .format(vector))
        cvss_object = cls(base_vector)
        if cvss_object.scores()[0] == score_value:
            return cvss_object
        else:
            raise CVSS3RHScoreDoesNotMatch('CVSS3 vector in Red Hat notation "{0}" has score of '
                                           '"{1}" which does not match specified score of "{2}"'
                                           .format(base_vector, cvss_object.scores()[0],
                                                   score))

    def __init__(self, vector):
        """
        Args:
            vector (str): string specifying CVSS3 vector, fields may be out of order, fields which
                          are not mandatory may be missing
        """
        self.vector = vector
        self.minor_version = None
        self.metrics = {}
        self.original_metrics = None
        self.missing_metrics = []

        self.scope = None
        self.modified_scope = None
        self.base_score = None
        self.temporal_score = None
        self.environmental_score = None

        self.isc_base = None
        self.isc = None
        self.esc = None
        self.modified_isc_base = None
        self.modified_isc = None
        self.modified_esc = None

        self.parse_vector()
        self.check_mandatory()
        self.handle_scope()
        self.add_missing_optional()
        self.compute_base_score()
        self.compute_temporal_score()
        self.compute_environmental_score()

    def parse_vector(self):
        """
        Parses metrics from the CVSS3 vector.

        Raises:
            CVSS3MalformedError: if vector is not in expected format
        """
        if self.vector == '':
            raise CVSS3MalformedError('Malformed CVSS3 vector, vector is empty')

        if self.vector.endswith('/'):
            raise CVSS3MalformedError('Malformed CVSS3 vector, trailing "/"')

        # Handle 'CVSS:3.x' in the beginning of vector and split vector
        if self.vector.startswith('CVSS:3.0/'):
            self.minor_version = 0
        elif self.vector.startswith('CVSS:3.1/'):
            self.minor_version = 1
        else:
            raise CVSS3MalformedError('Malformed CVSS3 vector "{0}" is missing mandatory prefix '
                                      'or uses unsupported CVSS version'.format(self.vector))

        try:
            fields = self.vector.split('/')[1:]
        except IndexError:
            raise CVSS3MalformedError('Malformed CVSS3 vector "{0}"'.format(self.vector))

        # Parse fields
        for field in fields:
            if field == '':
                raise CVSS3MalformedError('Empty field in CVSS3 vector "{0}"'.format(self.vector))

            try:
                metric, value = field.split(':')
            except ValueError:
                raise CVSS3MalformedError('Malformed CVSS3 field "{0}"'.format(field))

            if metric in METRICS_ABBREVIATIONS:
                if value in METRICS_VALUES[metric]:
                    if metric in self.metrics:
                        raise CVSS3MalformedError('Duplicate metric "{0}"'.format(metric))
                    self.metrics[metric] = value
                else:
                    raise CVSS3MalformedError('Unknown value "{0}" in field "{1}"'.format(value,
                                                                                          field))
            else:
                raise CVSS3MalformedError('Unknown metric "{0}" in field "{1}"'.format(metric,
                                                                                       field))

    def handle_scope(self):
        """
        Sets scope and modified scope variables based on S and MS metrics
        """
        self.scope = self.metrics['S']
        self.modified_scope = self.metrics.get('MS', None)
        if self.modified_scope in [None, 'X']:
            self.modified_scope = self.scope

    def check_mandatory(self):
        """
        Checks if mandatory fields are in CVSS3 vector.

        Raises:
            CVSS3MandatoryError: if mandatory metric is missing in the vector
        """
        missing = []
        for mandatory_metric in METRICS_MANDATORY:
            if mandatory_metric not in self.metrics:
                missing.append(mandatory_metric)
        if missing:
            raise CVSS3MandatoryError('Missing mandatory metrics "{0}"'.format(', '.join(missing)))

    def add_missing_optional(self):
        """
        Adds missing optional parameters, so they match the mandatory ones. Original metrics are
        also stored, as they may be used for printing back the minimal vector.
        """
        self.original_metrics = copy.copy(self.metrics)
        for abbreviation in ['MAV', 'MAC', 'MPR', 'MUI', 'MC', 'MI', 'MA']:
            if abbreviation not in self.metrics or self.metrics[abbreviation] == 'X':
                self.metrics[abbreviation] = self.metrics[abbreviation[1:]]

    def get_value(self, abbreviation):
        """
        Gets value of specific metric specified by its abbreviation. Handles exception for
        Privileges Required based on Scope or Modified scope.
        """
        string_value = self.metrics.get(abbreviation, 'X')
        if ((abbreviation == 'PR' and self.scope == 'C') or
                (abbreviation == 'MPR' and self.modified_scope == 'C')):
            result = {'X': None, 'N': D('0.85'), 'L': D('0.68'), 'H': D('0.50')}[string_value]
        else:
            result = METRICS_VALUES[abbreviation][string_value]
        return result

    def compute_isc_base(self):
        """
        ISCBase = 1 - [(1-ImpactConf) x (1-ImpactInteg) x (1-ImpactAvail)]
        """
        self.isc_base = D('1') - ((D('1') - self.get_value('C')) *
                                  (D('1') - self.get_value('I')) *
                                  (D('1') - self.get_value('A')))

    def compute_isc(self):
        """
        Scope Unchanged 6.42 x ISCBase
        Scope Changed 7.52 x [ISCBase-0.029] - 3.25 x [ISCBase-0.02]^15
        """
        if self.scope == 'U':
            self.isc = D('6.42') * self.isc_base
        elif self.scope == 'C':
            self.isc = (D('7.52') * (self.isc_base - D('0.029')) -
                        D('3.25') * (self.isc_base - D('0.02')) ** D('15'))
        else:  # This should never happen
            raise RuntimeError('Invalid Scope: "{0}"'.format(self.scope))

    def compute_esc(self):
        """
        8.22 x AttackVector x AttackComplexity x PrivilegeRequired x UserInteraction
        """
        self.esc = (D('8.22') * self.get_value('AV') * self.get_value('AC') *
                    self.get_value('PR') * self.get_value('UI'))

    def compute_base_score(self):
        """
        If (Impact sub score =< 0) 0 else,
        Scope Unchanged Round up (Minimum [(Impact + Exploitability),10])
        Scope Changed Round up (Minimum [1.08 x (Impact + Exploitability),10])
        """
        self.compute_isc_base()
        self.compute_isc()
        self.compute_esc()

        if self.isc <= D('0.0'):
            self.base_score = D('0.0')
        else:
            assert self.scope in ('U', 'C')
            if self.scope == 'U':
                self.base_score = round_up(min((self.isc + self.esc), D('10')))
            elif self.scope == 'C':
                self.base_score = round_up(min(D('1.08') * (self.isc + self.esc), D('10')))

    def compute_temporal_score(self):
        """
        Round up(BaseScore x ExploitCodeMaturity x RemediationLevel x ReportConfidence)
        """
        self.temporal_score = round_up(self.base_score * self.get_value('E') *
                                       self.get_value('RL') * self.get_value('RC'))

    def compute_modified_isc_base(self):
        """
        ISCModified = Minimum[[1-(1-M.IConf x CR)x(1-M.IInteg x IR)x(1-M.IAvail x AR)],0.915]
        """
        # Had to rename this term, as Modified Impact Sub score is already taken.
        self.modified_isc_base = min((D('1') - (D('1') - self.get_value('MC') *
                                                self.get_value('CR')) *
                                               (D('1') - self.get_value('MI') *
                                                self.get_value('IR')) *
                                               (D('1') - self.get_value('MA') *
                                                self.get_value('AR'))
                                      ),
                                     D('0.915'))

    def compute_modified_isc_30(self):
        """
        This is CVSS:3.0 version

        If Modified Scope Unchanged    6.42 x [ISCModified]
        If Modified Scope Changed      7.52 x [ISCModified-0.029] - 3.25 x [ISCModified-0.02]^15
        """
        if self.modified_scope == 'U':
            self.modified_isc = D('6.42') * self.modified_isc_base
        else:  # Modified scope has always value, if not defined then matches Scope
            self.modified_isc = (D('7.52') * (self.modified_isc_base - D('0.029')) -
                                 D('3.25') * (self.modified_isc_base - D('0.02')) ** D('15'))

    def compute_modified_isc(self):
        """
        This is CVSS:3.1 version

        If Modified Scope Unchanged    6.42 x [ISCModified]
        If Modified Scope Changed      7.52 x (ISCModified - 0.029) - 3.25 x
                                       (ISCModified x 0.9731 - 0.02)^13
        """
        if self.modified_scope == 'U':
            self.modified_isc = D('6.42') * self.modified_isc_base
        else:  # Modified scope has always value, if not defined then matches Scope
            self.modified_isc = (D('7.52') * (self.modified_isc_base - D('0.029')) -
                                 D('3.25') * (self.modified_isc_base * D('0.9731') - D('0.02'))
                                 ** D('13'))

    def compute_modified_esc(self):
        """
        8.22 x M.AttackVector x M.AttackComplexity x M.PrivilegeRequired x M.UserInteraction
        """
        self.modified_esc = (D('8.22') * self.get_value('MAV') * self.get_value('MAC') *
                             self.get_value('MPR') * self.get_value('MUI'))

    def compute_environmental_score(self):
        """
        If (Modified Impact Sub score =< 0)    0 else,
        If Modified Scope Unchanged    Round up(Round up (Minimum [
                          (M.Impact + M.Exploitability),10])
                          x Exploit Code Maturity
                          x Remediation Level
                          x Report Confidence)

        If Modified Scope Changed    Round up(Round up (Minimum [1.08
                          x (M.Impact + M.Exploitability),10])
                          x Exploit Code Maturity
                          x Remediation Level
                          x Report Confidence))
        """
        self.compute_modified_isc_base()
        if self.minor_version == 0:
            self.compute_modified_isc_30()
        else:
            self.compute_modified_isc()
        self.compute_modified_esc()

        if self.modified_isc <= D('0.0'):
            self.environmental_score = D('0.0')
        else:
            if self.modified_scope == 'U':
                modified = round_up(min((self.modified_isc + self.modified_esc), D('10')))
            else:
                modified = round_up(min(D('1.08') * (self.modified_isc + self.modified_esc), D('10')))
            self.environmental_score = round_up(modified *
                                                self.get_value('E') *
                                                self.get_value('RL') *
                                                self.get_value('RC'))

    def scores(self):
        """
        Returns all computed scores.

        Returns:
            (tuple of floats): Base Score, Temporal Score, Environmental Score
        """
        return float(self.base_score), float(self.temporal_score), float(self.environmental_score)

    def clean_vector(self, output_prefix=True):
        """
        Returns vector without optional metrics marked as X and in preferred order.

        Args:
            output_prefix (bool): defines if CVSS vector should be printed with prefix

        Returns:
            (str): cleaned CVSS3 with metrics in correct order
        """
        vector = []
        for metric in METRICS_ABBREVIATIONS:
            if metric in self.original_metrics:
                value = self.original_metrics[metric]
                if value != 'X':
                    vector.append('{0}:{1}'.format(metric, value))
        if output_prefix:
            prefix = 'CVSS:3.{0}/'.format(self.minor_version)
        else:
            prefix = ''
        return prefix + '/'.join(vector)

    def severities(self):
        """
        Returns severities based on scores.

        Returns:
            (tuple): Base Severity, Temporal Severity, Environmental Severity as strings
        """
        severities = []
        for score in (self.base_score, self.temporal_score, self.environmental_score):
            if score == D('0.0'):
                severities.append('None')
            elif score <= D('3.9'):
                severities.append('Low')
            elif score <= D('6.9'):
                severities.append('Medium')
            elif score <= D('8.9'):
                severities.append('High')
            else:
                severities.append('Critical')
        return tuple(severities)

    def rh_vector(self):
        """
        Returns cleaned vector with score in Red Hat notation, e.g. score/vector.

        Example: 6.5/CVSS:3.0/AV:P/AC:H/PR:H/UI:R/S:C/C:H/I:H/A:N/E:H/RL:O/RC:R/CR:H/MAC:H/MC:L
        """
        return str(self.scores()[0]) + '/' + self.clean_vector()

    def __eq__(self, o):
        if isinstance(o, CVSS3):
            return self.clean_vector().__eq__(o.clean_vector())
        return NotImplemented

    def __hash__(self):
        return hash(self.clean_vector())



# Copyright (c) 2016 Red Hat, Inc.
# Author: Stanislav Kontar, Red Hat Product Security
# License: LGPLv3+

"""
All exceptions needed by CVSS library.
"""


class CVSSError(Exception):
    """
    General CVSS exception.
    """
    pass


class CVSS2Error(CVSSError):
    """
    General CVSS2 exception.
    """
    pass


class CVSS2MalformedError(CVSS2Error):
    """
    Exception for malformed input CVSS2 vectors.
    """
    pass


class CVSS2MandatoryError(CVSS2Error):
    """
    Exception for missing mandatory fields.
    """
    pass


class CVSS2RHScoreDoesNotMatch(CVSS2Error):
    """
    Exception when parsing CVSS2 vectors in Red Hat notation, which have score not matching the
    computed score.
    """
    pass


class CVSS2RHMalformedError(CVSS2Error):
    """
    Exception for malformed input CVSS2 vectors in Red Hat notation.
    """
    pass


class CVSS3Error(CVSSError):
    """
    General CVSS3 exception.
    """
    pass


class CVSS3MalformedError(CVSS3Error):
    """
    Exception for malformed input CVSS3 vectors.
    """
    pass


class CVSS3MandatoryError(CVSS3Error):
    """
    Exception for missing mandatory fields.
    """
    pass


class CVSS3RHScoreDoesNotMatch(CVSS3Error):
    """
    Exception when parsing CVSS3 vectors in Red Hat notation, which have score not matching the
    computed score.
    """
    pass


class CVSS3RHMalformedError(CVSS3Error):
    """
    Exception for malformed input CVSS3 vectors in Red Hat notation.
    """
    pass







import codecs
from os import path

from setuptools import setup, find_packages


try:
    from collections import OrderedDict
except ImportError:
    dependencies = ['ordereddict']
else:
    dependencies = []

here = path.abspath(path.dirname(__file__))

with codecs.open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='cvss',
    version='2.1',
    description='CVSS2/3 library with interactive calculator for Python 2 and Python 3',
    long_description=long_description,
    url='https://github.com/skontar/cvss',
    author='Stanislav Kontar, Red Hat Product Security',
    author_email='skontar@redhat.com',
    license='LGPLv3+',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Security',
        'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords='security cvss score calculator',
    packages=find_packages(),
    install_requires=dependencies,
    entry_points={
        'console_scripts': [
            'cvss_calculator = cvss.cvss_calculator:main',
        ],
    },
)



# Copyright (c) 2016 Red Hat, Inc.
# Author: Stanislav Kontar, Red Hat Product Security
# License: LGPLv3+

"""
Constants for CVSS2 computations and checks. Generated using util/generate_constants.py.
"""

from __future__ import unicode_literals

from decimal import Decimal as D

try:
    from collections import OrderedDict
except ImportError:
    # noinspection PyUnresolvedReferences
    from ordereddict import OrderedDict


METRICS_ABBREVIATIONS = OrderedDict([('AV', 'Access Vector'),
                                     ('AC', 'Access Complexity'),
                                     ('Au', 'Authentication'),
                                     ('C', 'Confidentiality Impact'),
                                     ('I', 'Integrity Impact'),
                                     ('A', 'Availability Impact'),
                                     ('E', 'Exploitability'),
                                     ('RL', 'Remediation Level'),
                                     ('RC', 'Report Confidence'),
                                     ('CDP', 'Collateral Damage Potential'),
                                     ('TD', 'Target Distribution'),
                                     ('CR', 'Confidentiality Requirement'),
                                     ('IR', 'Integrity Requirement'),
                                     ('AR', 'Availability Requirement'),
                                     ])

METRICS_MANDATORY = ['AV', 'AC', 'Au', 'C', 'I', 'A']

METRICS_VALUES = {'AV': {'L': D('0.395'), 'A': D('0.646'), 'N': D('1')},
                  'AC': {'H': D('0.35'), 'M': D('0.61'), 'L': D('0.71')},
                  'Au': {'M': D('0.45'), 'S': D('0.56'), 'N': D('0.704')},
                  'C': {'N': D('0'), 'P': D('0.275'), 'C': D('0.660')},
                  'I': {'N': D('0'), 'P': D('0.275'), 'C': D('0.660')},
                  'A': {'N': D('0'), 'P': D('0.275'), 'C': D('0.660')},
                  'E': {'U': D('0.85'), 'POC': D('0.9'), 'F': D('0.95'), 'H': D('1'), 'ND': D('1')},
                  'RL': {'OF': D('0.87'), 'TF': D('0.90'), 'W': D('0.95'), 'U': D('1'), 'ND': D('1')},
                  'RC': {'UC': D('0.9'), 'UR': D('0.95'), 'C': D('1'), 'ND': D('1')},
                  'CDP': {'N': D('0'), 'L': D('0.1'), 'LM': D('0.3'), 'MH': D('0.4'), 'H': D('0.5'), 'ND': D('0')},
                  'TD': {'N': D('0'), 'L': D('0.25'), 'M': D('0.75'), 'H': D('1'), 'ND': D('1')},
                  'CR': {'L': D('0.5'), 'M': D('1'), 'H': D('1.51'), 'ND': D('1')},
                  'IR': {'L': D('0.5'), 'M': D('1'), 'H': D('1.51'), 'ND': D('1')},
                  'AR': {'L': D('0.5'), 'M': D('1'), 'H': D('1.51'), 'ND': D('1')},
                  }

METRICS_VALUE_NAMES = OrderedDict([('AV', OrderedDict([('L', 'Local'),
                                                       ('A', 'Adjacent Network'),
                                                       ('N', 'Network')])),
                                   ('AC', OrderedDict([('H', 'High'),
                                                       ('M', 'Medium'),
                                                       ('L', 'Low')])),
                                   ('Au', OrderedDict([('M', 'Multiple'),
                                                       ('S', 'Single'),
                                                       ('N', 'None')])),
                                   ('C', OrderedDict([('N', 'None'),
                                                      ('P', 'Partial'),
                                                      ('C', 'Complete')])),
                                   ('I', OrderedDict([('N', 'None'),
                                                      ('P', 'Partial'),
                                                      ('C', 'Complete')])),
                                   ('A', OrderedDict([('N', 'None'),
                                                      ('P', 'Partial'),
                                                      ('C', 'Complete')])),
                                   ('E', OrderedDict([('U', 'Unproven'),
                                                      ('POC', 'Proof-of-Concept'),
                                                      ('F', 'Functional'),
                                                      ('H', 'High'),
                                                      ('ND', 'Not Defined')])),
                                   ('RL', OrderedDict([('OF', 'Official Fix'),
                                                       ('TF', 'Temporary Fix'),
                                                       ('W', 'Workaround'),
                                                       ('U', 'Unavailable'),
                                                       ('ND', 'Not Defined')])),
                                   ('RC', OrderedDict([('UC', 'Unconfirmed'),
                                                       ('UR', 'Uncorroborated'),
                                                       ('C', 'Confirmed'),
                                                       ('ND', 'Not Defined')])),
                                   ('CDP', OrderedDict([('N', 'None'),
                                                        ('L', 'Low'),
                                                        ('LM', 'Low-Medium'),
                                                        ('MH', 'Medium-High'),
                                                        ('H', 'High'),
                                                        ('ND', 'Not Defined')])),
                                   ('TD', OrderedDict([('N', 'None'),
                                                       ('L', 'Low'),
                                                       ('M', 'Medium'),
                                                       ('H', 'High'),
                                                       ('ND', 'Not Defined')])),
                                   ('CR', OrderedDict([('L', 'Low'),
                                                       ('M', 'Medium'),
                                                       ('H', 'High'),
                                                       ('ND', 'Not Defined')])),
                                   ('IR', OrderedDict([('L', 'Low'),
                                                       ('M', 'Medium'),
                                                       ('H', 'High'),
                                                       ('ND', 'Not Defined')])),
                                   ('AR', OrderedDict([('L', 'Low'),
                                                       ('M', 'Medium'),
                                                       ('H', 'High'),
                                                       ('ND', 'Not Defined')])),
                                   ])



# Copyright (c) 2016 Red Hat, Inc.
# Author: Stanislav Kontar, Red Hat Product Security
# License: LGPLv3+

"""
Computes CVSS2/3 score either supplied on command line or interactively. By default it only takes
mandatory metrics and computes CVSS3.1.
"""

from __future__ import print_function

import argparse

from cvss import CVSSError, ask_interactively


PAD = 24  # string padding for score names


def main():
    try:
        parser = argparse.ArgumentParser(description=__doc__)
        parser.add_argument('-2', action='store_true', help='compute CVSS2 instead')
        parser.add_argument('-3', action='store_true', help='compute CVSS3.0 instead')
        parser.add_argument('-a', '--all', action='store_true', help='ask for all metrics')
        parser.add_argument('-v', '--vector', help='input string with CVSS vector')
        parser.add_argument('-n', '--no-colors', action='store_true',
                            help='do not use terminal coloring')
        args = parser.parse_args()

        # Import the correct CVSS module
        if getattr(args, '2'):
            version = 2
            from cvss import CVSS2 as CVSS
        else:
            if getattr(args, '3'):
                version = 3.0
            else:
                version = 3.1
            from cvss import CVSS3 as CVSS

        # Vector input, either from command line or interactively
        if args.vector:
            vector_string = args.vector
        else:
            vector_string = ask_interactively(version, args.all, args.no_colors)

        # Compute scores and clean vector
        try:
            cvss_vector = CVSS(vector_string)
        except CVSSError as e:
            print(e)
        else:
            scores = cvss_vector.scores()
            if version == 2:
                print('CVSS2')
                severities = None
            elif version >= 3.0:
                print('CVSS3')
                severities = cvss_vector.severities()
            else:
                raise ValueError('Unknown CVSS version: {0}'.format(version))

            for i, score_name in enumerate(['Base Score', 'Temporal Score', 'Environmental Score']):
                print(score_name + ':' + ' ' * (PAD - len(score_name) - 2), end='')

                if version >= 3.0:
                    print(scores[i], '({0})'.format(severities[i]))
                else:
                    print(scores[i])
            print('Cleaned vector:       ', cvss_vector.clean_vector())
            print('Red Hat vector:       ', cvss_vector.rh_vector())
    except (KeyboardInterrupt, EOFError):
        print()


if __name__ == '__main__':
    main()
